import base64
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicNumbers

def jwk_to_rsa_pem(jwk):
    def decode_value(val):
        decoded = base64.urlsafe_b64decode(ensure_bytes(val) + b"==")
        return int.from_bytes(decoded, "big")

    def ensure_bytes(key):
        if isinstance(key, str):
            key = key.encode("utf-8")
        return key

    return (
        RSAPublicNumbers(n=decode_value(jwk["n"]), e=decode_value(jwk["e"]))
        .public_key(default_backend())
        .public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )
    )



import requests


def tenant_metadata_endpoint(tenant_id) -> str:
    template = (
        "https://login.microsoftonline.com"
        "/{TENANT_ID}/v2.0/.well-known/openid-configuration"
    )
    format_args = {
        "TENANT_ID": tenant_id,
    }
    return template.format(**format_args)


class OpenIdDiscovery:
    verify: bool = True
    metadata_endpoint_template = (
        "https://login.microsoftonline.com"
        "/{TENANT_ID}/v2.0/.well-known/openid-configuration"
    )

    def __init__(self, metadata_endpoint: str):
        self.metadata_endpoint = metadata_endpoint

    def get_configuration(self) -> dict:
        response = requests.get(self.metadata_endpoint, verify=self.verify)
        try:
            response.raise_for_status()
        except requests.HTTPError as error:
            raise self.RetrieveError() from error
        return response.json()

    def get_keys(self) -> dict:
        url = self.get_configuration()["jwks_uri"]
        response = requests.get(url, verify=self.verify)
        try:
            response.raise_for_status()
        except requests.HTTPError as error:
            raise self.RetrieveError() from error
        return response.json()

    def get_key(self, key_id: str) -> str:
        for key in self.get_keys()["keys"]:
            if key.get("kid") == key_id:
                return key
        raise self.UnknownKeyError(key_id)

    class RetrieveError(Exception):
        pass

    class UnknownKeyError(Exception):
        pass
